import axios from 'axios'
import _ from 'underscore'
import store from './store'
import * as qs from 'qs'

// apiHost，用于api直接返回的页面。
// 不能直接取host，因为default.admin.ivydad.com的apiHost为admin.ivydad.com
export function getApiHost() {
  let index = window.location.host.indexOf('admin.ivydad.com')
  return window.location.host.slice(Math.max(index, 0))
}

export function setTitle(title) {
  title = title ? title + ' - Home' : 'iView project'
  window.document.title = title
};

let alerted = false
export function alertOnce(message) {
  if (alerted) return
  alerted = true
  alert(typeof message === 'string' ? message : JSON.stringify(message))
}

export function fetchJson(url, opt) {
  let error = window.app.$Message && window.app.$Message.error.bind(window.app.$Message) || alert
  opt = opt || {}
  opt.paramsSerializer = params => qs.stringify(params)
  if (opt && opt.data) {
    opt.method = opt.method || 'POST'
  }
  return axios(url, opt).then(r => {
    let data = r.data
    if (data.code && !opt.silent) {
      error(data.message)
      return Promise.reject(data)
    }
    return data
  }, err => {
    let serr = typeof err === 'string' ? err : JSON.stringify(err)
    if (serr.indexOf('ECONNREFUSED') >= 0) {
      error('网络错误，请确认服务器是否正常提供服务')
    } else if (serr.indexOf('ServiceUnavailableError') >= 0) {
      error('服务不可用')
    } else {
      alertOnce(err.response.data)
      error('服务器错误')
      console.error('error fetching:', url, _.omit(opt, 'paramsSerializer'))
      console.error('error:', err.response.data)
    }
    return Promise.reject(err)
  })
}

export async function beforeEnter(to, from, next) {
  let logined = await fetchJson('/api/v1/user/user/info', { silent: 1})
  if (logined.code && logined.code === 403) {
    window.location.href = `${window.config.loginUrl}?redirect=${encodeURIComponent(window.location.href)}`
  } else if (logined.code) {
    window.app.$Message.error(logined.message)
  } else {
    // 把后端返回的userAccess转成方便使用的形式
    let access = logined.userAccess = _.reduce(logined.userAccess, (acc, r) => {
      acc[r.subsystem] = [].concat(acc[r.subsystem], JSON.parse(r.auth || '[]'))
      return acc
    }, {})
    _.extend(store, logined)
    // let access = store.userAccess
    let subsystem = getSubSystem()
    let accSub = access[subsystem] || []
    store.menus = window.routes.filter(e => e.title && (access['*'].includes('*') || accSub.includes('*') || accSub.includes(e.path)))
    next()
  }
}

export function getSubSystem() {
  return window.location.origin.split('.')[0].split('//')[1] || 'default'
}

export function deleteEmptyStringKey(obj) {
  return _.object(_.pairs(obj).filter(p => p[1]))
}

export function sleep(delay) {
  return new Promise((resolve, reject) => {
    setInterval(resolve, delay)
  })
}

export function getFileContent(file) {
  return new Promise((resolve, reject) => {
    let reader = new FileReader()
    reader.onload = e => {
      resolve(e.target.result)
    }
    reader.readAsBinaryString(file)
  })
}

export async function getExcelTable(file) {
  let cont = await getFileContent(file)
  let xlsx = window.XLSX.read(cont, {type: 'binary'})
  let sheet = xlsx.Sheets[xlsx.SheetNames[0]]
  let [, colStart, rowStart, colEnd, rowEnd] = sheet['!ref'].match(/([A-Z])(\d+):([A-Z]+)(\d+)/)
  colEnd = colEnd.length > 1 ? 'Z' : colEnd
  let table = []
  for (let r = parseInt(rowStart); r <= parseInt(rowEnd); r++) {
    let row = {}
    for (let c = colStart; c.charCodeAt(0) <= colEnd.charCodeAt(0); c = String.fromCharCode(c.charCodeAt(0) + 1)) {
      row[c.charCodeAt(0) - colStart.charCodeAt(0)] = sheet[c + r] && sheet[c + r].w || ''
    }
    table.push(row)
  }
  return table
}

export function getFileContentArrayBuffer(file) {
  return new Promise((resolve, reject) => {
    let reader = new FileReader()
    reader.onload = e => {
      resolve(e.target.result)
    }
    reader.readAsArrayBuffer(file)
  })
}

export function dateAgo(secondsOffset, format = '') {
  return new Date(new Date().getTime() + secondsOffset * 1000).format(format)
}

export function updateLoading(promise, update) {
  if (!promise.then) return
  update(true)
  promise.then(r => {
    update(false)
    return r
  }, err => {
    update(false)
    return Promise.reject(err)
  })
}

export function guid() {
  function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1)
  }
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
    s4() + '-' + s4() + s4() + s4()
}

export function showThisRoute(menuAccess, userAccess) {
  for (let access of userAccess) {
    if (access && access.indexOf(menuAccess) === 0) {
      return true
    }
  }
  return false
}
// 获取32位uuid
export function uuid() {
  let s = []
  let hexDigits = "0123456789abcdef"
  for (let i = 0; i < 32; i++) {
    s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1)
  }
  s[14] = "4" // bits 12-15 of the time_hi_and_version field to 0010
  s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1)  // bits 6-7 of the clock_seq_hi_and_reserved to 01
  s[8] = s[13] = s[18] = s[23]

  let uuid = s.join("")
  return uuid
}
// 校验下拉框
export function createValidator(fn) {
  return (rule, value, callback) => {
    let error = fn(value, rule)
    callback(error ? new Error(error) : undefined)
  }
}

export let validatorNotEmpty = createValidator(value => value || value === 0 ? undefined : "请选择")
// 获取n小时后的时间
export function getHoursAfter(date, hours) {
  return new Date(date.setHours(date.getHours() + hours))
}
// 获取n天后的时间
export function getDateAfter(date, days) {
  return new Date(date.setDate(date.getDate() + days))
}

export function validateForm(form) {
  return new Promise(res => {
    form.validate(valid => res(valid))
  })
}
// 表单中输入为int类型的<input>的校验
export const validateNumber = (rule, value, callback) => {
  if (value === '') {
    callback(new Error('该项不能为空'))
  } else if (isNaN(value)) {
    callback(new Error('输入有误，请输入数字。'))
  } else {
    callback()
  }
}

/**
 * ES7扩展 padStart
 * @param {*} str
 * @param {*} len
 * @param {*} pad
 */
export const padStart = (str, len, pad) => {
  while (str.length < len) {
    str = pad + str
  }
  return str
}

/**
 * 将秒转换成音频需要的格式
 * @param  {Number} duration [description]
 * @return {[type]}          [description]
 */
export const activeDuration = (duration = 0) => {
  let a = []
  for (let i = 0; i < 2; i++) {
    a[i] = '' + Math.floor(duration % 60)
    duration = duration / 60
  }
  return a.reverse().map(e => padStart(e, 2, '0')).join(':')
}
/**
 * 秒格式化
 * @param {*} s
 */
export const secondFormate = (s = 0) => {
  let a = ""
  a += Math.floor(s / 60) > 0 ? `${padStart(Math.floor(s / 60) + '', 2, '0')}′` : ''
  a += `${padStart(Math.floor(s % 60) + '', 2, '0')}″`
  return a
}
export function trim(str) {
  return str.replace(/(^[ \t\n\r]+)|([ \t\n\r]+$)/g, '')
}
